/// IMPORTING THIRD PARTY PACKAGES
import 'package:flutter/cupertino.dart';
import 'package:provider/provider.dart';

/// IMPORTING LOCAL PACKAGES
import 'package:zwap_design_system/zwap_design_system.dart';

import 'components/components.dart';


/// The state of the network card with the changeNotifier class
class NetworkCardState extends ChangeNotifier{

  /// Elements inside this network card
  final List<TupleType<DateTime, User>> elements;

  /// The pageNumber to fetch more data
  final int pageNumber;

  /// Callback function to fetch more data
  final Function() fetchMoreData;

  /// Search filter controller text
  final TextEditingController searchFilterController;

  /// Private note controller text
  final TextEditingController privateNoteController;

  NetworkCardState({
    required this.elements,
    required this.pageNumber,
    required this.fetchMoreData,
    required this.searchFilterController,
    required this.privateNoteController
  });

}


/// Custom widget to display the network card
class NetworkCard extends StatelessWidget{

  /// The network stats to show inside this
  final NetworkStats networkStats;

  /// The current logged user
  final User currentUser;

  NetworkCard({Key? key,
    required this.networkStats,
    required this.currentUser
  }): super(key: key);


  @override
  Widget build(BuildContext context) {
    return CustomCard(
      childComponent: VerticalScroll(
        childComponent: Column(
          children: [
            Padding(
              padding: EdgeInsets.symmetric(horizontal: 20),
              child: Column(
                children: [
                  HeaderNetwork(firstName: this.currentUser.firstName, lastName: this.currentUser.lastName, baseStateType: [
                    BaseStateType(baseStatTitle: Utils.translatedText("totalMeeting"),
                        baseStatNumber: this.networkStats.meetings.toString(),
                        iconData: IconData(Utils.iconData("cameraIcon"), fontFamily: "zwapIcon")),
                    BaseStateType(baseStatTitle: Utils.translatedText("totalStreak"),
                        baseStatNumber: this.networkStats.streak.toString(),
                        iconData: IconData(Utils.iconData("cameraIcon"), fontFamily: "zwapIcon")),
                    BaseStateType(baseStatTitle: Utils.translatedText("invitedFriends"),
                        baseStatNumber: this.networkStats.invitedFriends.toString(),
                        iconData: IconData(Utils.iconData("cameraIcon"), fontFamily: "zwapIcon")),
                  ]),
                  Padding(
                    padding: EdgeInsets.all(10),
                    child: GeneralCardStat(title: Utils.translatedText("zwapScoreTitle"),
                        stat: this.networkStats.zwapScore.toString(),
                        cardType: GeneralStatType.TagStat),
                  ),
                  Padding(
                    padding: EdgeInsets.all(10),
                    child: GeneralCardStat(title: Utils.translatedText("generatedMeetingsTitle"),
                        stat: "${this.networkStats.generatedMeetings.toString()} ${Utils.translatedText("meetings")}",
                        cardType: GeneralStatType.TagText),
                  ),
                  Padding(
                    padding: EdgeInsets.all(10),
                    child: GeneralCardStat(title: Utils.translatedText("zwapPoints"),
                        stat: "${this.networkStats.zwapPoints.toString()} ${Utils.translatedText("zwapPoints")}",
                        cardType: GeneralStatType.TagText),
                  ),
                  Padding(
                    padding: EdgeInsets.all(20),
                    child: BaseText(
                      texts: ["${this.networkStats.totalContacts} ${Utils.translatedText("contactsTitle")}"],
                      baseTextsType: [BaseTextType.subTitle],
                      textsColor: [DesignColors.greyPrimary],
                    ),
                  ),
                  Consumer<NetworkCardState>(
                    builder: (builder, provider, child){
                      return Column(
                        children: [
                          Padding(
                            padding: EdgeInsets.symmetric(horizontal: 20, vertical: 10),
                            child: Filters(
                              searchInputController: provider.searchFilterController,
                              showFiltersCallBack: () => {},
                              exportContactsCallBack: () => {},
                              importContactsCallBack: () => {},
                              searchCallBack: (String searchKey) => {},
                            ),
                          ),
                          InfiniteScroll<TupleType<DateTime, User>>(
                              elements: provider.elements,
                              pageNumber: provider.pageNumber + 1,
                              fetchMoreData: () => provider.fetchMoreData(),
                              dynamicWidget: (TupleType<DateTime, User> element) => NetworkItemCard(
                                userData: element.b,
                                lastMeeting: element.a,
                                privateNoteController: provider.privateNoteController,
                              )
                          )
                        ],
                      );
                    }
                  )
                ],
              ),
            ),
          ],
        ),
      ),
    );
  }



}