/// IMPORTING THIRD PARTY PACKAGES
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:provider/single_child_widget.dart';

/// IMPORTING LOCAL PACKAGES
import 'package:zwap_design_system/zwap_design_system.dart';


class DynamicQuestionState extends ChangeNotifier{

  final Map<String, dynamic> bodyData = {};

  final List<ProfileQuestions> profileQuestions;

  DynamicQuestionState({required this.profileQuestions});

  void handleChanges(String key, dynamic newData){
    this.bodyData[key] = newData;
    notifyListeners();
  }

  Map<String, CustomDropDownState> questionsMapping(){
    Map<String, CustomDropDownState> questionsMapping = {};
    this.profileQuestions.forEach((element) {
      if(element.questionOne.questionType == QuestionType.InputDropdown){
        questionsMapping[element.questionOne.questionTitleValue] = CustomDropDownState();
      }
      if(element.questionTwo != null && element.questionTwo!.questionType == QuestionType.InputDropdown){
        questionsMapping[element.questionTwo!.questionTitleValue] = CustomDropDownState();
      }
    });
    return questionsMapping;
  }

  Map<String, dynamic> getBodyValues(){
    Map<String, dynamic> tmp = {};
    this.questionsMapping().forEach((key, value) {
      tmp[key] = value.dropdownValue;
    });
    this.bodyData.addAll(tmp);
    return this.bodyData;
  }

  List<SingleChildWidget> getMultiProviders(){
    List<SingleChildWidget> multiProviders = [];
    Map<String, CustomDropDownState> mappings = this.questionsMapping();
    mappings.forEach((key, value) {
      multiProviders.add(
        ChangeNotifierProvider<CustomDropDownState>(
          create: (_) => value,
        )
      );
    });
    return multiProviders;
  }

}

/// Custom widget to display the edit profile card
class DynamicQuestion extends StatelessWidget{

  /// Check if the current dynamic question has a validator function or not
  bool _hasValidator(QuestionElement questionElement){
    return questionElement.questionType != QuestionType.InputDropdown && questionElement.regexValidate != null;
  }

  /// Validate input
  bool validateInput(dynamic value, QuestionElement questionElement){
      if(this._hasValidator(questionElement)){
        return Utils.validateRegex(questionElement.regexValidate!, value);
      }
      return true;
  }

  /// Define a const padding inside this form input
  EdgeInsets _constPadding(){
    return EdgeInsets.symmetric(horizontal: 10);
  }

  /// Retrieve the question type and the type of the widget to insert the data into this form
  Widget _getQuestionType(QuestionElement questionElement, DynamicQuestionState provider){
    TextEditingController controller = TextEditingController();
    controller.addListener(() {
      provider.handleChanges(questionElement.questionTitleValue, controller.text);
    });
    bool hasValidator = this._hasValidator(questionElement);
    switch(questionElement.questionType){
      case QuestionType.InputNumber:
        return Padding(
          padding: this._constPadding(),
          child: BaseInput(
            hasValidatorIcon: hasValidator,
            handleValidation: (dynamic value) => hasValidator ? this.validateInput(value, questionElement) : null,
            placeholderText: questionElement.questionPlaceholder,
            inputType: InputType.inputNumber,
            controller: controller,
          ),
        );
      case QuestionType.InputDropdown:
        return Padding(
          padding: this._constPadding(),
          child: ChangeNotifierProvider<CustomDropDownState>(
            create: (_) => provider.questionsMapping()[questionElement.questionTitleValue]!,
            builder: (builder, child){
              return CustomDropDown(
                values: questionElement.questionOptions!,
                defaultValue: questionElement.questionDefaultValue,
              );
            }
          ),
        );
      case QuestionType.InputArea:
        return Padding(
          padding: this._constPadding(),
          child: BaseInput(
            hasValidatorIcon: hasValidator,
            handleValidation: (dynamic value) => hasValidator ? this.validateInput(value, questionElement) : null,
            inputType: InputType.inputArea,
            maxLines: 3,
            placeholderText: questionElement.questionPlaceholder,
            controller: controller,
          ),
        );
      default:
        return Padding(
          padding: this._constPadding(),
          child: BaseInput(
            hasValidatorIcon: hasValidator,
            handleValidation: (dynamic value) => hasValidator ? this.validateInput(value, questionElement) : null,
            inputType: InputType.inputArea,
            placeholderText: questionElement.questionPlaceholder,
            controller: controller,
          ),
        );
    }
  }

  /// Retrieve current question inside the edit profile form
  Widget _getCurrentQuestion(ProfileQuestions profileQuestions, DynamicQuestionState provider){
    List<Widget> tmp = [];
    Widget questionOne = Expanded(
      child: LabelWidget(labelText: profileQuestions.questionOne.questionTitle,
          paddingInside: this._constPadding(),
          childComponent: this._getQuestionType(profileQuestions.questionOne, provider)),
    );
    tmp.add(questionOne);
    if(profileQuestions.questionTwo != null){
      Widget questionTwo = Expanded(
        child: LabelWidget(labelText: profileQuestions.questionTwo!.questionTitle,
            paddingInside: this._constPadding(),
            childComponent: this._getQuestionType(profileQuestions.questionTwo!, provider)),
      );
      tmp.add(questionTwo);
    }
    return Padding(
      padding: EdgeInsets.symmetric(horizontal: 80, vertical: 10),
      child: Row(children: tmp),
    );
  }


  @override
  Widget build(BuildContext context) {
    DynamicQuestionState provider = context.read<DynamicQuestionState>();
    return MultiProvider(
      providers: provider.getMultiProviders(),
      child: Column(
        children: [
          for (var i = 0; i < provider.profileQuestions.length; i++) this._getCurrentQuestion(provider.profileQuestions[i], provider),
        ],
      ),
    );
  }

}