/// IMPORTING THIRD PARTY PACKAGES
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

/// IMPORTING LOCAL PACKAGES
import 'package:zwap_design_system/zwap_design_system.dart';

/// The custom input style in base of the current type
enum InputType{
  inputText,
  inputNumber,
  inputSearch,
  inputArea,
  inputEmail,
  inputPassword,
  inputPhone,
  inputAddress,
  inputUrl,
  inputButton
}

/// Custom input state
class BaseInput extends StatefulWidget{
  
  /// The placeholder text to show in the input text
  final String placeholderText;

  /// The custom input type
  final InputType inputType;

  /// Text controller for handling the value inside without a provider
  final TextEditingController controller;

  /// Boolean flag to show or not the validation icon
  final bool hasValidatorIcon;

  /// The max lines to rendering in the input text multi lines
  final int? maxLines;

  /// The button text inside
  final String? buttonText;

  /// Custom function callBack to handle click on custom input with type == InputButton
  final Function()? baseInputButtonCallBack;

  /// Handle the validation with a custom callBack
  final Function(dynamic value)? handleValidation;


  BaseInput({Key? key,
    required this.placeholderText,
    required this.inputType,
    required this.controller,
    this.hasValidatorIcon = false,
    this.maxLines,
    this.buttonText,
    this.baseInputButtonCallBack,
    this.handleValidation,
  }) : super(key: key){
    if(this.inputType == InputType.inputButton){
      assert(this.buttonText != null, "Button text could not be null on input type == InputButton");
      assert(this.baseInputButtonCallBack != null , "Base input callback function could not be null on input type == InputButton ");
    }
    else{
      assert(this.buttonText == null, "Button text must be null on input type != InputButton");
      assert(this.baseInputButtonCallBack == null , "Base input callback function must be null on input type != InputButton ");
    }
  }
  
  _BaseInputState createState() => _BaseInputState();

}


/// Custom widget to rendering custom input
class _BaseInputState extends State<BaseInput> {

  /// Bool validator to handle if the user is writing
  bool _isWriting = false;

  /// Bool validator to check if the user wrote everything correctly
  bool _isValid = false;
  
  /// It retrieves the input style
  TextStyle _getTextInputStyle(){
    return TextStyle(
      fontFamily: "",
      fontSize: 13,
      color: DesignColors.blackPrimary
    );
  }

  /// Changing the state inside when user starts to write
  void changeState(bool value){
    if (!_isWriting){
      setState(() {
        this._isWriting = true;
        Future.delayed(Duration(seconds: 2)).whenComplete((){
          this._isWriting = false;
        });
      });
    }
  }

  /// Validate the user's input
  void validateCallBack(dynamic value){
    if(widget.handleValidation != null){
      setState(() {
        this._isValid = widget.handleValidation!(value);
      });
    }
  }


  /// It retrieves the suffix icon
  Widget? _getSuffixIcon(){
    if(widget.hasValidatorIcon){
      if(this._isWriting && !this._isValid){
        return SizedBox(
          height: 15,
          width: 15,
          child: CircularProgressIndicator(),
        );
      }
      if(this._isValid){
        return Icon(
          Icons.done,
          color: DesignColors.greenPrimary,
          size: 15,
        );
      }
    }
    return null;
  }


  /// The input field decoration for the search input
  InputDecoration _searchInputFieldDecoration(){
    return InputDecoration(
      focusedBorder: OutlineInputBorder(
        borderSide: BorderSide(color: DesignColors.greyPrimary, width: 1.0),
      ),
      enabledBorder: OutlineInputBorder(
        borderSide: BorderSide(color: DesignColors.greyPrimary, width: 1.0),
      ),
      prefixIcon: Icon(Icons.search, size: 21),
      suffix: this._getSuffixIcon()
    );
  }


  /// The field decoration for any text field
  InputDecoration _getTextFieldDecoration(String hintText){
    return new InputDecoration(
      focusedBorder: OutlineInputBorder(
        borderRadius: widget.inputType == InputType.inputButton ? BorderRadius.circular(ConstantsValue.radiusValue) : const BorderRadius.all(Radius.circular(4.0)),
        borderSide: BorderSide(color: DesignColors.greyPrimary, width: 1.0),
      ),
      enabledBorder: OutlineInputBorder(
        borderRadius: widget.inputType == InputType.inputButton ? BorderRadius.circular(ConstantsValue.radiusValue) : const BorderRadius.all(Radius.circular(4.0)),
        borderSide: BorderSide(color: DesignColors.greyPrimary,  width: 1.0),
      ),
      hintText: hintText,
      hintStyle: TextStyle(
        color: DesignColors.greyPrimary
      ),
      suffix: widget.inputType == InputType.inputButton ? null : this._getSuffixIcon()
    );
  }

  /// It retrieves the input type in base of the params
  TextInputType _getInputType(){
    switch(widget.inputType){
      case InputType.inputAddress:
        return TextInputType.streetAddress;
      case InputType.inputArea:
        return TextInputType.multiline;
      case InputType.inputNumber:
        return TextInputType.number;
      case InputType.inputText:
        return TextInputType.text;
      case InputType.inputSearch:
        return TextInputType.text;
      case InputType.inputEmail:
        return TextInputType.emailAddress;
      case InputType.inputPassword:
        return TextInputType.text;
      case InputType.inputPhone:
        return TextInputType.phone;
      case InputType.inputUrl:
        return TextInputType.url;
      case InputType.inputButton:
        return TextInputType.text;
    }
  }

  /// It retrieves the text field inside this custom input field
  Widget _getTextInputField(){
    return TextField(
        controller: widget.controller,
        keyboardType: this._getInputType(),
        maxLines: widget.maxLines ?? 1,
        textCapitalization: TextCapitalization.sentences,
        cursorColor: DesignColors.bluePrimary,
        obscureText: widget.inputType == InputType.inputPassword,
        textAlign: TextAlign.start,
        style: this._getTextInputStyle(),
        onEditingComplete: () => this.changeState(false),
        onChanged: (dynamic value) => {
          this.validateCallBack(value),
          this.changeState(true),
        },
        decoration: widget.inputType == InputType.inputSearch ? this._searchInputFieldDecoration() :
        this._getTextFieldDecoration(widget.placeholderText)
    );
  }

  /// It retrieves the button inside this custom input
  Widget _getBaseButton(){
    return Container(
      transform: Matrix4.translationValues(-5.0, 0.0, 0.0),
      child: BaseButton(
          buttonText: widget.buttonText!,
          buttonTypeStyle: ButtonTypeStyle.baseInputButton,
          onPressedCallback: () => widget.baseInputButtonCallBack,
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: Row(
        children: [
          Flexible(child: this._getTextInputField()),
          widget.inputType == InputType.inputButton ?
          Flexible(child: SizedBox(
            child: this._getBaseButton(),
            height: 48,
          )) : Container()
        ],
      ),
    );
  }
}
